/********************************************************************/
/*                                                                  */
/*   Program Name :  speana                                         */
/*                                                                  */
/*      Version:  2.52  2018-05-17                                  */
/*                                                                  */
/*      Copyright (c) 2002-2018 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
D:\IPVLBI\data\k5vssp32\kas11ft32a.dat 21 1 0 K5/VSSP32 2
  Spectrum analysis of IP-VLBI board data
        (off-line processing)

          T.Kondo 2002-05-20
               Ver 0.00 2002-05-20 
               Ver 1.41 2002-10-01 18000->16000 bug fix
               Ver 1.42 2002-10-11 set default frequency axis linear
               Ver 1.43 2004-10-11 support sampling Frequency > 16MHz
               Ver 1.44 2005-06-12 update due to update in common header name
               Ver 1.45 2005-09-27 Allow to use PGDISP as an environmetal variable
                                   for PPGPLOT device
               Ver 2.00 2005-10-28 Improve processing speed drastically
                                   support K5/VSSP32 data processing
                                   FFTW3.0 is supported if hope so.
               Ver 2.10 2006-01-31 support linear power axis
               Ver 2.11 2007-02-21 update due to libvssp.h update
                                   (Ver2.10 didn't run with new libvssp32.h)
               Ver 2.12 2007-10-26 leave window open in case of Windows
               Ver 2.13 2007-11-02 support GNUPLOT
               Ver 2.14 2007-11-04 improve appearance in case of GNUPLOT postscript
               Ver 2.15 2008-02-19 allow file name starting with '0' and having more than 
                                   equal to 2 characters.
                                   options '--help' and '--ver[sion]' were added
               Ver 2.16 2011-02-07 "long int" type is changed to "int" type
               Ver 2.17 2014-06-13 minor update (add YYYY/DDD in title in case of VSSP32)
               Ver 2.18 2014-08-12 allow start offset in floating second
               Ver 2.19 2014-09-11 minor debug
               Ver 2.20 2014-10-29 add plot frequency range and amplitude range as run parameters
               Ver 2.21 2015-10-23 support AUX field format #22
               Ver 2.22 2015-10-28 separate libspeplot as an independent library
                                   add new operator interface (options)
               Ver 2.23 2015-11-03 minor update
               Ver 2.24 2017-02-09 minor update to use the same libspeplot library under corr directory
               Ver 2.30 2017-02-16 update to use FFT results as defined way (time->frequency)
               Ver 2.40 2018-02-04 debug in case of 1 ch mode. Serious bug was introduced.
               Ver 2.41 2018-02-18 debug "-m" option.  "-m" changes to "-mode"
               Ver 2.42 2018-02-19 add option '-ps psname' for set postscript file other than pgplot.ps
               Ver 2.50 2018-04-09 add options '-noinfo', '-velocity baserfmhz refmhz'
                                   add members " flag_no_info", "flag_velocity", "rfmhz", "refmhz" to 
                                   runp_t variable
               Ver 2.51 2018-04-10 add options '-nohead', '-nohead1', '-nohead2'
                                   add members "flag_no_title", "mode_title" to runp_t variable
               Ver 2.52 2018-05-15 add members "minlin", "maxlin", "tbar_P1_mW", and "tbar_P2_K" to runp_t variable
                                   add options '-linmin minlin' and '-linmax maxlin' to set maximum and menimum by 
                                   linear power
                                   add option '-tbar p1 p2' to display temperature scale bar in the graph in case of 
                                   linear power plot mode.
               Ver 2.53 2018-05-16 add option '-tpos xpos ypos' to set X (0.0-1.0) and Y(0.0-1.0) positions of 
                                   temperature scale in the graph (default xpos=0.8 ypos=0.75).
                                   add members "tbar_xpos", "tbar_ypos" to runp_t variable
               Ver 2.54 2018-05-17 add option '-y_old' to set Y (power) axis to old style 


    How to run:

       speana filename [mode [sekibun [pmode [comment [soffset [f1khz [f2khz [mindbm [maxdbm]]]]]]]]]

         or

       speana filename [options]

        where filename -- data file name
                            0 means "tds.data" file
               mode -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale (default)
                          Y: amplitude axis mode
                             =0: log scale (default)  
                             =1: linear scale (common for 4 channels)
                             =2: linear scale (each max and min)
                      -value: auto correlation (value point center)
                      -1:  auto correlation (full range)
                      
               sekibun  -- integration period (sec)
                            default is 1.0
               pmode -- plot device mode
                       0: XTERM and PostScript file out (default)
                       1: PostScript out only 
                       2: XTERM only 
               comment  -- comment (displayed at upper side of graph)
                           (do not include space)
                           If omitted, you will be asked later. <= no more asked as of 2015-10-28
                           NOTE: In the interactive mode, you can use space.
               soffset -- start offset (sec) (default is 0.0)
               f1khz  ---  lower frequency of power max search and plot range (kHz).  default is 0
               f2khz  ---  highest frequency of power max search and plot range (kHz).  default is max video frequency
               mindbm  ----  display power minimum (dBm) (default auto)\n");
               maxdbm  ----  display power maximum (dBm) (default auto)\n");

       [options]
               -mode mode
               -pmode pmode
               -se[kibun] sekibun
               -ti[ntegration] sekibun
               -i[ntegration] sekibun
               -c[omment] comment
               -so[ffset] soffset
               -to[ffset] soffset
               -o[ffset] soffset
               -f1[khz] f1khz
               -f2[khz] f2khz
               -min[dbm] mindbm
               -max[dbm] maxdbm
               -minlin minlin  --   set display power minimum by antilog number (default auto)  2018-05-15  
               -maxlin maxlin  --   set display power maximum by antilog number (default auto)  2018-05-15
               -nops    --  same as "-p 2"

               -ps file_name  -- set POSTSCRIPT file name other than "pgplot.ps" 

               -noinfo  -- suppress processing date and file information on a graph
               -velocity basemhz refmhz  
                        -- set velocity axis instead of frequency axis
                           basemhz : baseband RF frequency (MHz) -VE means LSB
                           refmhz : reference frequency (MHz) where velocity=0
                                          velocity is computed as follows
                                          V=C*df/f0
                                             where C: light velocity
                                                   f0: reference frequency
                                                   df: frequency from f0 
               -nohead|-notitle  -- suppress title in a graph
               -nohead1|-notitle1 -- suppress time info in title
               -nohead2|-notitle2 -- suppress sampling and time information 

               -tbar p1 p2  ---  draw temperature bar in the figure.
                                          length given by p1 in milliwat and its temperature
                                          given by 2nd parameter p2 in kelvin
               -tpos xpos ypos ---  set X (0.0-1.0) and Y(0.0-1.0) positions of the
                                   temperature scale in the graph (default xpos=0.8 ypos=0.75)
               -y_old  ---  set numbering of Y (power) axis to old style (number is parallel to the axis)


               *** following is not implemented yet **

               -h[alt] --  halt after each plot (default only one plot)
               -sm[sec] sleep_msec -- set suspend time(msec) to next plot
                                   other than default
             

*/
#define SPEANA_VERSION "Ver. 2018-05-16"

//#define USE_GNUPLOT    // use GNUPLOT  (default is use of PGPLOT)

#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#include "libvssp32.h"
#include "libspeplot.h"

/*    Offset from dB to dBm conversion from A/D multibit sampling data */ 
#define OFFSET1  16.0     //1 bit sampling full power convert to 0 dB (theoretical)
      /* following constants are provisional  (measured on 2002/5/18) */
#define OFFSET2   9.0     //2 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET4  -1.0     //4 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET8 -26.0     //8 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET16 -78.0     //16 bit sampling data offset (this is for USBADC adjusting value) 2015-10-27

#define DEFLT_DATA_FILE "tds.data"   // default data file name

//#define DEFLT_SLEEP_MSEC 100        // default sleep suspend time in msec

//#ifdef WIN32
//int tempcounter;  // counter for tempfile manage
//#endif


typedef struct runp     // run parameters for this
{
    char fname[128];  // data file name
    char comment[60]; // comment
    char psname[128]; // PS file name (default pgplot.ps   2018-02-19
    double sekibun;   // given integration period
    double soffset;   // start offset in floating second 
    double f1khz;     // search frequency range (lower boundary) (kHz)
    double f2khz;     // search frequency range (upper boundary) (kHz)
    double mindbm;    // plot amplitude range in the case of log scale
    double maxdbm;    //     "
    double minlin;    // plot amplitude range in the case of linear scale 2018-05-15
    double maxlin;    //     "
    int  mode2;        // axis mode
    int  mode;        // axis mode
    int  ymode;       //
    int amode;        // 
    int pmode;        // plot mode
    int flag_halt;
    int sleep_msec;
    int flag_no_info;        // 2018-04-09  suppres display of file name anfnd processing date
    int flag_velocity;       // 2018-04-09  display velocity instead of frequency
    int flag_no_title;       // 2018-04-10  title display flag 1:no display 0: display title
    int mode_title;       // 2018-04-10 title display mode 0: full 1: omit time info: 2: omit both sampling and time
    double rfmhz;  // 2018-04-09 base band RF frequency (MHz) (used in case of flag_velocity=1) -VE means LSB
    double refmhz; // 2018-04-09 RF frequency where set velocity = 0  (used in case of flag_velocity=1)
    double tbar_p1_mW;        // 2018-05-15  temperature parameter P1
    //double tbar_p2_K;    // 2018-05-15  temperature parameter P2    
    char tbar_p2_K[10];    // 2018-05-15  temperature parameter P2    
    double tbar_xpos;        // 2018-05-16  X position (0.0-1.0) of temperature scale
    double tbar_ypos;        // 2018-05-16  Y position (0.0-1.0) of temperature scale
    int ystyle_old;     // 2018-05-17   1:  set numbering of Y (Power) axis to old style (parallel to the axis) 
} runp_t;


void print_usage1(speana_param_t *st)
{
    char pgdevice_dflt[20];
    printf("speana %s",SPEANA_VERSION);
#ifdef USE_FFTW
    printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
    printf(" compliled for GNUPLOT \n");
    strcpy(pgdevice_dflt,G_DISPLAY);
#else
    printf(" compliled for PGPLOT \n");
    strcpy(pgdevice_dflt,st->g_display);
#endif
    printf("\n");
    printf("Usage: speana filename [mode [sekibun [pmode [comment [soffset [f1khz [f2khz [mindbm [maxdbm]]]]]]]]]\n");
    printf("         or\n");
    printf("       speana filename [options]\n");
    printf("\n");
    printf("         where  filename ---- Data file name\n");
    printf("                              if 0 is given, tds.data will be used\n");
    printf("                mode -------- set axis mode or autocorrelation mode\n");
    printf("                              =0 : amplitude log, frequency log\n");
    printf("                              =1 (deflt) : amplitude log, frequency linear\n");
    printf("                              =10 : amplitude linear, frequency log\n");
    printf("                              =11 : amplitude linear, frequency linear\n");
    printf("                              =-1 : Auto correlation function plot\n");
    printf("                                     instead of spectrum\n");
    printf("                              =-N : N points auto correlation function plot\n");
    printf("                sekibun --- Averaging period (sec)\n");
    printf("                             default is 1.0\n");
    printf("                pmode ---- Plot device selection\n");
    printf("                           =0 :  \"%s\" and PostScript file (%s) out\n",pgdevice_dflt,st->ps_fname);
    printf("                           =1 :  PostScript file (%s) out only\n",st->ps_fname);
    printf("                           =2 :  \"%s\" only\n",pgdevice_dflt);
    printf("                          default is 0\n");
    printf("                comment --- Comment appeared on a graph (do not use ' '(space))\n");
    //printf("                            if omitted, you will be asked later.\n");
    printf("                soffset --- Start time offset in sec\n");
    printf("                             default is 0.0\n"); 
    printf("                f1khz   ----  search frequency range (lower boundary) (kHz)\n");
    printf("                              for power Maximum (default is 0.0)\n");
    printf("                f2khz   ----  search frequency range (upper boundary) (kHz)\n");
    printf("                              for power Maximum (default is video frequency)\n");
    printf("                mindbm  ----  display amplitude minimum (dBm) (default auto)\n");
    printf("                maxdbm  ----  display amplitude maximum (dBm) (default auto)\n");
    printf("\n");
    printf("            [options]\n");
    printf("                -mode mode\n");
    printf("                -pmode pmode\n");
    printf("                -se[kibun] sekibun\n");
    printf("                -ti[ntegration] sekibun\n");
    printf("                -i[ntegration] sekibun\n");
    printf("                -c[omment] comment\n");
    printf("                -so[ffset] soffset\n");
    printf("                -to[ffset] soffset\n");
    printf("                -o[ffset] soffset\n");
    printf("                -f1[khz] f1khz\n");
    printf("                -f2[khz] f2khz\n");
    printf("                -min[dbm] mindbm\n");
    printf("                -max[dbm] maxdbm\n");
    printf("                -linmin  minlin --  set display power minimum by antilog number\n");
    printf("                -linmax  maxlin --  set display power maximum by antilog number\n");
    printf("                -nops  --- same as \"-p 2\" (no postscript out)\n");
    printf("                -ps psname  --- set postscript file name other than pgplot.ps\n");
    printf("                -noinfo  -- suppress processing date and file information\n");
    printf("                -velocity baserfmhz refmhz\n");
    printf("                        -- set velocity axis instead of frequency axis\n");
    printf("                           baserfmhz : baseband RF frequency (MHz) -VE means LSB\n");
    printf("                           refmhz : reference frequency (MHz) where velocity=0\n");
    printf("                -nohead | -notitle  -- suppress title in a graph\n");
    printf("                -nohead1 | -notitle1 -- suppress time info in title\n");
    printf("                -nohead2 | -notitle2 -- suppress sampling and time information \n");
    printf("                -tbar p1 p2  ---  draw temperature bar in the figure.\n");
    printf("                                  length given by p1 (mW) and its temperature\n");
    printf("                                  given by 2nd parameter p2 (K)\n");
    printf("                -tpos xpos ypos ---  set X (0.0-1.0) and Y(0.0-1.0) positions of\n"); 
    printf("                                     the temperature scale in the graph\n");  
    printf("                                     (default xpos=0.8 ypos=0.75)\n");  
    printf("                -y_old  ---  set numbering of Y (Power) axis to old style\n"); 
    printf("                             (number is parallel to the axis)\n"); 
//    printf("                -h[alt]  --- halt after each plot (default only one plot)\n");
//    printf("                -sm[sec] sleep_msec  --- suspend time (msec) after each plot\n");
//    printf("                                         in case of continuous mode (default 100msec)\n");
    printf("\n");
#ifndef USE_GNUPLOT   // using gnuplot
    printf("\n");
    printf("  Note: %s ---  environment variable for PGPLOT display device\n",PGDISP);
    printf("                   = %s",getenv(PGDISP));
    if(getenv(PGDISP)==NULL) {
        printf(",  program default is ( %s )\n",PGDISP_DFLT);
    } else {
        printf("\n");
    }
#endif
    printf("\n");
    exit(0);
}

void runparam_get1(int argc, char *argv[],runp_t *runp)                       
{
    /* get run parameters type 1 (conventional run tyep) */

    //int n,i;
    char cbuf[256];
    char cbuf2[40];    // 2018-04-09

	// initialize parameters

    runp->fname[0]='\0';
    runp->comment[0]='\0';
    runp->soffset=0.0;
    runp->sekibun=1.0;
    runp->mode2=1;
    runp->mode=1;  // default frequency axis mode (=Linear scale)
    runp->ymode=0; // default amplitude axis mode (=LOG scale)
    runp->amode=0; // auto correlation mode off
    runp->pmode=0; // default plot mode
    runp->f1khz=0.0;
    runp->f2khz=0.0;
    runp->mindbm=0.0;
    runp->maxdbm=0.0;
    runp->minlin=99.0;   // 2018-05-15
    runp->maxlin=99.0;   // 2018-05-15
    runp->flag_halt=0;
    runp->sleep_msec=0;
    strcpy(runp->psname,PS_OUT_FILE);   // 2018-02-19

    runp->flag_no_info=0;  // 2018-04-09
    runp->flag_velocity=0; // 2018-04-09
    runp->rfmhz=0.0;       // 2018-04-09
    runp->refmhz=0.0;      // 2018-04-09
    runp->flag_no_title=0;   // 2018-04-10
    runp->mode_title=0;      // 2018-04-10

    runp->tbar_p1_mW=0.0;    // 2018-05-15
    //runp->tbar_p2_K=0.0;    // 2018-05-15
    runp->tbar_p2_K[0]='\0'; // 2018-05-15
    runp->tbar_xpos=0.8;     // 2018-05-16
    runp->tbar_ypos=0.75;     // 2018-05-16
    runp->ystyle_old=0;     // 2018-05-17

    if (argc>=2){
        strcpy(runp->fname,*(argv+1));
    }
    if (strlen(*(argv+1))==1 && runp->fname[0]=='0'){
          strcpy(runp->fname, DEFLT_DATA_FILE);
    }
    
    /* check 2nd parameter is old type or new option type */
    // 2nd parameter can be negative integer 
    // so two step judging is adopted to avoid negative value like "-90" as judged 
    // as an option command like "-mode XX"

	if (argc>=3 && strncmp(*(argv+2),"-",1)==0){ 
        // further check whether it is not numerical one 
        strcpy(cbuf, *(argv+2));
        if(cbuf[1] < 48 || cbuf[1] > 57) {  //   this is not numerical '0' ... '9'   
           goto bypass;
        }
    }

    /* plot mode set from operator */
    if (argc>=3){
       runp->mode2 = atoi(*(argv + 2 ));
       if(runp->mode2 <0 ){
            runp->amode=-runp->mode2;
            runp->mode=0;
       } else {
            runp->amode=0;
            runp->ymode=runp->mode2/10;
            runp->mode=runp->mode2 % 10;
       }
    }

    if (argc>=4){
       runp->sekibun = atof(*(argv + 3 ));
    }

    if (argc>=5){
       runp->pmode = atoi(*(argv + 4 ));
    }

    /* get comment */

    if (argc>=6){
          strcpy(runp->comment,*(argv+5));
    } else {

          /* enter any comment */

//          printf("Enter any comment -> ");
//          read1line(runp->comment);      // 1 line read from stdin
    }
    if (argc>=7){
       runp->soffset = atof(*(argv + 6 ));
    }

	if (argc>=8){
	   runp->f1khz = atof(*(argv + 7 ));
    }
	if (argc>=9){
	   runp->f2khz = atof(*(argv + 8 ));
    }
	if (argc>=10){
        runp->mindbm = atof(*(argv + 9 ));
        runp->minlin = pow(10.0,runp->mindbm/10.0);   // 2018-05-15
    }
	if (argc>=11){
	   runp->maxdbm = atof(*(argv + 10 ));
       runp->maxlin = pow(10.0,runp->maxdbm/10.0);   // 2018-05-15
    }
    goto exit2return;

bypass:

    //if(cmnd_get(argc,argv,"-m",cbuf)==0){                 // -mode 
    if(cmnd_get(argc,argv,"-mode",cbuf)==0){                 // -mode   2018-02-18
       runp->mode2 = atoi(cbuf);  
       if(runp->mode2 <0 ){
            runp->amode=-runp->mode2;
            runp->mode=0;
       } else {
            runp->amode=0;
            runp->ymode=runp->mode2/10;
            runp->mode=runp->mode2 % 10;
       }
    }
    //if(cmnd_get(argc,argv,"-p",cbuf)==0) runp->pmode = atoi(cbuf);     // -pmode 
    if(cmnd_get(argc,argv,"-pmode",cbuf)==0) runp->pmode = atoi(cbuf);     // -pmode 
    if(cmnd_get(argc,argv,"-c",cbuf)==0) strcpy(runp->comment,cbuf);   // -comment 
    if(cmnd_get(argc,argv,"-se",cbuf)==0) runp->sekibun = atof(cbuf);  // -sekibun 
    if(cmnd_get(argc,argv,"-ti",cbuf)==0) runp->sekibun = atof(cbuf);  // -tinteg 
    if(cmnd_get(argc,argv,"-i",cbuf)==0) runp->sekibun = atof(cbuf);   // -integration 
    if(cmnd_get(argc,argv,"-so",cbuf)==0) runp->soffset = atof(cbuf);  // -soffset 
    if(cmnd_get(argc,argv,"-to",cbuf)==0) runp->soffset = atof(cbuf);  //  -toffset 
    if(cmnd_get(argc,argv,"-o",cbuf)==0) runp->soffset = atof(cbuf);   //  -offset 
    if(cmnd_get(argc,argv,"-f1",cbuf)==0) runp->f1khz = atof(cbuf);    //  -f1khz 
    if(cmnd_get(argc,argv,"-f2",cbuf)==0) runp->f2khz = atof(cbuf);    //  -f2khz 
    if(cmnd_get(argc,argv,"-min",cbuf)==0){
        runp->mindbm = atof(cbuf);  //  -mindbm 
        runp->minlin = pow(10.0,runp->mindbm/10.0);   // 2018-05-15
    }
    if(cmnd_get(argc,argv,"-max",cbuf)==0){
        runp->maxdbm = atof(cbuf);  //  -maxdbm 
        runp->maxlin = pow(10.0,runp->maxdbm/10.0);   // 2018-05-15
    }
    if(cmnd_get(argc,argv,"-linmin",cbuf)==0){   // 2018-05-15
        runp->minlin = atof(cbuf);  //   
        runp->mindbm = 10.0*log10(runp->minlin);
    }
    if(cmnd_get(argc,argv,"-linmax",cbuf)==0){   // 2018-05-15
        runp->maxlin = atof(cbuf);  // 
        runp->maxdbm = 10.0*log10(runp->maxlin);
    }
    if(cmnd_get(argc,argv,"-nops",cbuf)==0){
        runp->pmode=2;
    }
    if(cmnd_get(argc,argv,"-h",cbuf)==0) runp->flag_halt=1;           // -halt
    if(cmnd_get(argc,argv,"-sm",cbuf)==0) runp->sleep_msec = atoi(cbuf);   //  -smsec 
    if(cmnd_get(argc,argv,"-ps",cbuf)==0) {     // -ps psname    2018-02-19
        if(cbuf[0] != '\0'){
            strcpy(runp->psname,cbuf); 
        }
    }
    if(cmnd_get(argc,argv,"-noin",cbuf)==0){                     // 2018-04-09
        runp->flag_no_info=1;
    }
    if(cmnd_get_n2para(argc, argv, "-vel", cbuf, cbuf2)==0){      // 2018-04-09
        runp->flag_velocity=1;
        runp->rfmhz=atof(cbuf);
        runp->refmhz=atof(cbuf2);
    }
    if(cmnd_get_np(argc,argv,"-nohead",cbuf)==0){                     // 2018-04-10
        runp->flag_no_title=1;
    }
    if(cmnd_get_np(argc,argv,"-nohead1",cbuf)==0){                     // 2018-04-10
        runp->mode_title=1;
    }
    if(cmnd_get_np(argc,argv,"-nohead2",cbuf)==0){                     // 2018-04-10
        runp->mode_title=2;
    }
    if(cmnd_get_np(argc,argv,"-notitle",cbuf)==0){                     // 2018-04-10
        runp->flag_no_title=1;
    }
    if(cmnd_get_np(argc,argv,"-notitle1",cbuf)==0){                     // 2018-04-10
        runp->mode_title=1;
    }
    if(cmnd_get_np(argc,argv,"-notitle2",cbuf)==0){                     // 2018-04-10
        runp->mode_title=2;
    }
    if(cmnd_get_n2para(argc, argv, "-tbar", cbuf, cbuf2)==0){      // 2018-05-15
        runp->tbar_p1_mW=atof(cbuf);
        //runp->tbar_p2_K=atof(cbuf2);
        strcpy(runp->tbar_p2_K,cbuf2);
    }
    if(cmnd_get_n2para(argc, argv, "-tpos", cbuf, cbuf2)==0){      // 2018-05-16
        runp->tbar_xpos=atof(cbuf);
        runp->tbar_ypos=atof(cbuf2);
    }
    if(cmnd_get_np(argc,argv,"-y_old",cbuf)==0){                   // 2018-05-17
        runp->ystyle_old=1;
    }


exit2return:;

}

void monit_param(runp_t *runp)
{
    printf("  fname   : %s\n",runp->fname);
    printf("  mode2   : %d\n",runp->mode2);
    printf("  mode    : %d\n",runp->mode);
    printf("  amode   : %d\n",runp->amode);
    printf("  ymode   : %d\n",runp->ymode);
    printf("  sekibun : %f\n",runp->sekibun);
    printf("  soffset : %f\n",runp->soffset);
    printf("  f1khz   : %f\n",runp->f1khz);
    printf("  f2khz   : %f\n",runp->f2khz);
    printf("  mindbm  : %f\n",runp->mindbm);
    printf("  maxdbm  : %f\n",runp->maxdbm);
}


int main(int argc, char *argv[])
{
    //int nloop,sekibun;
    //char cw;
    int nloop;
    int flag_last;   // 2015-10-27
    int i,imax, m, k, kaisu;
    int smode,usampl,nfft;
	int kbytes;
    int64_t bits, bytes;  // 2105-10-23
    int numb;
    //int amode;    // >0 auto correlation plot mode 
    //int mode;  // frequency axis mode  0: log  1: linear 
    //int mode2;  // 2 digits axis mode
    int ymode; // amplitude scale mode   0: log   1: linear
    int scount[4];
    //float fsampl,tsampl,fact;
    double fsampl,tsampl,fact;
    double *cdata;
    double *f_hanning;
    double theta;
    double sum,dc;
    float pows;
    float pmax,pmin,pmaxdbm,pmindbm,cofst;
    float ymaxdb,ymindb;
#ifdef USE_FFTW
    fftw_complex *cc;
    fftw_plan pfcc, pbcc;
#endif
    int isoffset;     // start offset in integer second      2014-08-12
    double fsoffset;  // fractional part of start offset     2014-08-12
    double ftime;     // time in second to judge start time  2014-08-12
    int start_flag;   // flag to judge start time            2014-08-12
    double nfact;
    double tinteg,tusampl;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    char ltime1st[60];
    char cbuf[20];
    //char pgdevice_dflt[20];
    //char* dflt_fname="tds.data";
    runp_t runp;
    K5VSSP_file_t K5;
    speana_param_t st;

    for(k=0; k<16; k++) st.chtable[k]=k+1;    // 2017-02-09 to keep compatibility with libspeplot under corr 

    /* set graphic device */ // moved to below run parameter get  2018-02-19
/*
    strcpy(st.ps_fname,PS_OUT_FILE);
#ifdef USE_GNUPLOT   // using gnuplot
    strcpy(st.g_display,G_DISPLAY);
    //strcpy(pgdevice_dflt,G_DISPLAY);
    strcpy(st.g_ps_device,G_PS_DEVICE);
#else
    strcpy(st.g_display,select_env_dflt(PGDISP, PGDISP_DFLT));
    //strcpy(pgdevice_dflt,st.g_display);
    sprintf(st.g_ps_device,"%s%s",st.ps_fname,G_PS_DEVICE);
#endif
*/
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){
         print_usage1(&st);
         //printf(" Hit return key for another how to run ");
         //cw=getchar();  /* dummy read to accept just return */
         //print_usage2();
         exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ 
          printf("speana %s",SPEANA_VERSION);
#ifdef USE_FFTW
          printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
          printf(" compliled for GNUPLOT \n");
#else
          printf(" compliled for PGPLOT \n");
#endif
          printf("\n");
          exit(0);
    }
    
    /* start processing */

     printf("speana %s",SPEANA_VERSION);
#ifdef USE_FFTW
     printf(" compliled with FFTW3.0 \n");
#endif
#ifdef USE_GNUPLOT   /* using gnuplot */
     printf(" compliled for GNUPLOT \n");
#else
     printf(" compliled for PGPLOT \n");
#endif
     printf("\n");

    /* get run parameters */
    runparam_get1(argc, argv, &runp);

    //monit_param(&runp); exit(0);  // debug

    /* set graphic device */  // moved from a head

    //strcpy(st.ps_fname,PS_OUT_FILE);
    strcpy(st.ps_fname,runp.psname);   // 2018-02-19
#ifdef USE_GNUPLOT   // using gnuplot
    strcpy(st.g_display,G_DISPLAY);
    //strcpy(pgdevice_dflt,G_DISPLAY);
    strcpy(st.g_ps_device,G_PS_DEVICE);
#else
    strcpy(st.g_display,select_env_dflt(PGDISP, PGDISP_DFLT));
    //strcpy(pgdevice_dflt,st.g_display);
    sprintf(st.g_ps_device,"%s%s",st.ps_fname,G_PS_DEVICE);
#endif

    /* copy runparameters to other variables */
    strcpy(st.fname,runp.fname);
    strcpy(K5.fname,st.fname);

    st.mode2=runp.mode2;
    st.mode=runp.mode;
    st.amode=runp.amode;
    ymode=runp.ymode;
    st.sekibun=runp.sekibun;

    st.pmode=runp.pmode;
    strcpy(st.comment,runp.comment);
    st.soffset=runp.soffset;

    isoffset=(int)st.soffset;       // 2014-08-12
    fsoffset=fmod(st.soffset,1.0);  // 2014-08-12
    if(fsoffset > 0.0){             // 2014-08-12
        start_flag=0;               // 2014-08-12
    } else {                        // 2014-08-12
        start_flag=0;               // 2014-08-12
    }                               // 2014-08-12
    ftime=0.0;                      // 2014-08-12

    st.f1khz=runp.f1khz;
    st.f2khz=runp.f2khz;
    st.mindbm=runp.mindbm;
    st.maxdbm=runp.maxdbm;
    st.minlin=runp.minlin;      // 2015-05-15
    st.maxlin=runp.maxlin;      // 2015-05-15

    st.flag_no_info=runp.flag_no_info;   // 2018-04-09
    st.flag_velocity=runp.flag_velocity; // 2018-04-09
    st.rfmhz=runp.rfmhz;                 // 2018-04-09
    st.refmhz=runp.refmhz;               // 2018-04-09
    st.flag_no_title=runp.flag_no_title;  // 2018-04-10
    st.mode_title=runp.mode_title;        // 2018-04-10

    st.tbar_p1_mW=runp.tbar_p1_mW;        // 2018-05-15
    //st.tbar_p2_K=runp.tbar_p2_K;          // 2018-05-15
    strcpy(st.tbar_p2_K,runp.tbar_p2_K);  // 2018-05-15
    st.tbar_xpos=runp.tbar_xpos;          // 2018-05-16
    st.tbar_ypos=runp.tbar_ypos;          // 2018-05-16
    st.ystyle_old=runp.ystyle_old;        // 2018-05-17

    cstart1sec = clock();

    printf("Data File is %s\n",K5.fname);
    K5.size=get_file_size(K5.fname);
    if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ){
         printf("File (%s) Open Error! \n",K5.fname);
        goto exitend2;
    }
    k5vssp_t_ini(&K5);

    /* Data header check to get sampling parameters */

    if((kbytes=CheckK5header_with_Read(&K5)) < 0){
        printf("File read error! \n");
        i=fclose( K5.stream ) ;
        goto exitend;
    }

    /* parameter set */

    flag_last=0;   // 2015-10-27

    st.sfreq=K5.sfreq;
    st.adbit=K5.adbit;
    st.numch=K5.numch;
    //fsampl=(float)(1000.0*(float)st.sfreq);   // sampling frequency (Hz)
    //tsampl=(float)(1.0/fsampl);            // sampling period (sec)
    fsampl=1000.0*(double)st.sfreq;   // sampling frequency (Hz)
    tsampl=1.0/fsampl;            // sampling period (sec)
    //bits=((long)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header
    //bits=((int)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header
    bits=((int64_t)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header  2015-10-23
    bytes=bits/8;                     // # of bytes in 1 sec data except header
    K5.fsample=fsampl;
    //sekibun=1;                       // Averaging period for Spectrum (sec)
    K5.bytesec=(int)(bits/8);                  //# of bytes in 1 sec data

    smode=1;                         // low resolution mode

    if(smode ==1){ 
        usampl=10000; 
    } else {
        usampl=50000;
        if(st.sfreq == 40)  usampl=40000; 
        if(st.sfreq == 100) usampl=100000; 
        if(st.sfreq == 200) usampl=200000; 
    }
    imax=(st.sfreq*1000)/usampl;
    numb=usampl*st.adbit*st.numch/8;   // # of bytes in unit # of samples
    K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
    K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second


    tusampl=(double)usampl/K5.fsample;   // period of usampl

    printf("imax numb  %d %d\n",K5.imax,K5.numb);    //debug

    nfft=1024;
    if(usampl < 524288) nfft= 524288;
    if(usampl < 262144) nfft= 262144;
    if(usampl < 131072) nfft= 131072;
    if(usampl < 65536)  nfft= 65536;
    if(usampl < 32768)  nfft= 32768;
    if(usampl < 16384)  nfft= 16384;

    st.nfft=nfft;

    //nfact=2.0/(double)nfft;
    nfact=1.0/(double)nfft;  // 2017-02-16

    K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl
    K5.tsample=1.0/K5.fsample;    // 2015-10-23
    K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

    K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer
    K5.idata  = (int *)malloc(32*K5.nword*sizeof(int)); // K5 data read buffer

#ifdef USE_FFTW
    cc=fftw_malloc(sizeof(fftw_complex)*nfft);

    pfcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_FORWARD, FFTW_ESTIMATE);
    pbcc=fftw_plan_dft_1d(nfft, cc, cc, FFTW_BACKWARD, FFTW_ESTIMATE);
#endif

    cdata = (double *)malloc(2*nfft*sizeof(double));
    //st.spows = (float *)malloc(4*nfft*sizeof(float));
    st.spows = (float *)malloc(st.numch*nfft*sizeof(float));  // 2018-02-04
    f_hanning = (double *)malloc(nfft*sizeof(double));

    //Hanning filter coeeficients prepare
    for(k=0; k<nfft; k++){
        theta=360.0*(double)k/(double)(nfft-1) - 180.0;
        theta=fmod(theta,360.0) ;
        f_hanning[k]=0.5*(1.0+cos(theta*PIRA));
    }

    //fact=((float)usampl)*((float)usampl);
    fact=(double)nfft/(double)usampl;  // 2017-02-16
    fact*=fact;                        // 2017-02-16

    /* foward startffset seconds */
    //if(fwdNsecK5(st.soffset, &K5) <0){
    if(fwdNsecK5(isoffset, &K5) <0){            // 2014-08-12
        //printf("Foward error %d\n",st.soffset);
        printf("Foward error %d\n",isoffset);
        i=fclose( K5.stream ) ;
        goto exitend;
    }

    //for(k=0; k<4*nfft; k++) st.spows[k]=0.0;
    //for(k=0; k<4; k++) scount[k]=0;
    for(k=0; k<st.numch*nfft; k++) st.spows[k]=0.0;   // 2018-02-04
    for(k=0; k<st.numch; k++) scount[k]=0;            // 2018-02-04

    printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
        K5.hh,K5.mm,K5.ss,K5.seconds);
    if(K5.format==2){  // VSSP32          2014-06/13
	   sprintf(ltime1st,"Time(UT) %04d/%03d %02d:%02d:%02d",
           K5.year,K5.totalday,K5.hh,K5.mm,K5.ss);
    } else {
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d",K5.hh,K5.mm,K5.ss);
    }
    if(fsoffset > 0.0){   // 2014-08-12
        sprintf(cbuf,".%03d",(int)(1000.0*fsoffset));
        strcat(ltime1st,cbuf);
    }
    strcpy(st.lab2,ltime1st);
    strcat(st.lab2,"  ");
    strcat(st.lab2,st.comment);
    
        
    tinteg=0.0;

    nloop=0;
loop1:
    if(nloop >0) {
        if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            i=fclose( K5.stream ) ;
            goto exitend;
        }
        printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
            K5.hh,K5.mm,K5.ss,K5.seconds);
    }

    for(i=0; i<K5.imax; i++){
        K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
        if(K5.wordread < K5.nword){  // probably no more data     2015-10-27
            flag_last=1;
            if(K5.wordread==0){
                tusampl=0;
                goto exithere;
            }
            /* recalc tusampl and fact for this reading */
            if(st.adbit*st.numch >= 8){
                usampl=K5.wordread*4/(st.adbit*st.numch/8);
                fact=((float)usampl)*((float)usampl);
                tusampl=(double)usampl/K5.fsample;   // period of usampl
            } else {
                tusampl=0;
                goto exithere;
            }
        }

        K5.pos+=K5.wordread*4;
        K5.vflag=1; // validiate data
        parseADdata_3N(&K5);

        if(start_flag==0){
            if(ftime < fsoffset){
                ftime+=tusampl;
                continue;
            } else {
                start_flag=1;
            }
        }


        for(m=0; m<st.numch; m++){
            kaisu=0;
            sum=0.0;

            // initialize array
#ifdef USE_FFTW
            for(k=0; k<nfft; k++){ cc[k][0]=0.0; cc[k][1]=0.0;}
#else
            for(k=0; k<2*nfft; k++) cdata[k]=0.0;
#endif

            for(k=0; k<K5.npos/K5.numch; k++){

                /* set data */
#ifdef USE_FFTW
                cc[k][0]=(double)K5.idata[k*st.numch+m];  
                sum+=cc[k][0];
                //if(k<10 && m==0) printf(" %f",(double)K5.idata[k*st.numch+m]); // debug
#else
                cdata[2*k]=(double)K5.idata[k*st.numch+m];  
                sum+=cdata[2*k];
#endif
                kaisu++;
            }
             
            dc=sum/(double)kaisu;    // DC component
     
            // DC offset removal from data
            for(k=0; k<K5.npos/st.numch; k++){
#ifdef USE_FFTW
                cc[k][0]-=dc;
#else
                cdata[2*k]-=dc;
#endif
            }

            // do hanning filtering
            if(st.amode ==0) {    // spectrum mode
                for(k=0; k<nfft; k++){
#ifdef USE_FFTW
                   cc[k][0]*=f_hanning[k];  // hanning filter
                   cc[k][1]*=f_hanning[k];  // hanning filter
#else
                   cdata[2*k]*=f_hanning[k];
                   cdata[2*k+1]*=f_hanning[k];
#endif
                }
                //hanning(cdata,usampl);  // hanning filter
            }

            // do forward FFT
#ifdef USE_FFTW
            fftw_execute(pfcc);
            for(k=0; k<nfft; k++){
                cc[k][0]*=nfact; cc[k][1]*=nfact;
            }
#else
            cdft(2*nfft,-1,cdata);
#endif
            // do adjustment according to actual sample number
            for(k=0; k<nfft; k++){
#ifdef USE_FFTW
                //pows=(float)((cc[k][0]*cc[k][0]+cc[k][1]*cc[k][1])/fact);
                pows=(float)((cc[k][0]*cc[k][0]+cc[k][1]*cc[k][1])*fact);                // 2017-02-16
#else
                //pows=(float)((cdata[2*k]*cdata[2*k]+cdata[2*k+1]*cdata[2*k+1])/fact);
                pows=(float)((cdata[2*k]*cdata[2*k]+cdata[2*k+1]*cdata[2*k+1])*fact);     // 2017-02-16
#endif
                //st.spows[k*4+m]=st.spows[k*4+m]+pows;
                st.spows[k*st.numch+m]=st.spows[k*st.numch+m]+pows;  // 2018-02-04
            }

            scount[m]++;
        }
exithere:
        tinteg+=tusampl;
        if(flag_last == 1){  // 2015-10-27
            if(tinteg==0.0) goto exitloop;
            if(st.sekibun > tinteg){
               st.sekibun=tinteg;
            }
            goto exitloop;  
        }
        if(tinteg >= st.sekibun) goto exitloop;

    }
    goto loop1;
exitloop:
    i=fclose( K5.stream ) ;

    if(st.amode > 0) {   // auto correlation function calc
       for(m=0; m<st.numch; m++){
          for(k=0; k<nfft; k++){
             //st.spows[k*4+m]/=scount[m]; // nomalization
             st.spows[k*st.numch+m]/=scount[m]; // nomalization   2018-02-04
#ifdef USE_FFTW
             //cc[k][0]=st.spows[k*4+m]; cc[k][1]=0.0;
             cc[k][0]=st.spows[k*st.numch+m]; cc[k][1]=0.0;  // 2018-02-04
#else
             //cdata[2*k]=st.spows[k*st.numch+m];  // 2018-02-04
             cdata[2*k]=st.spows[k*4+m];
             cdata[2*k+1]=0.0;
#endif
          }
          if(m==1 || m==3){
#ifdef USE_FFTW
              cc[0][0]=1.0;
#else
              cdata[0]=1.0;  // to avoid math error
#endif
          }
#ifdef USE_FFTW
          fftw_execute(pbcc);
#else
          cdft(2*nfft,1,cdata);
#endif
          for(k=0; k<nfft; k++){
             // if(k<10) printf("%d %e\n",k,cdata[2*k]);  // debug
             // if(k>nfft-10) printf("%d %e\n",k,cdata[2*k]);  // debug
#ifdef USE_FFTW
             if(k==0) pmax=(float)cc[0][0];
             if(cc[k][0] > pmax) pmax=(float)cc[k][0];
             //st.spows[k*4+m]=(float)cc[k][0];
             st.spows[k*st.numch+m]=(float)cc[k][0];   // 2018-02-04
#else
             if(k==0) pmax=(float)cdata[0];
             if(cdata[2*k] > pmax) pmax=(float)cdata[2*k];
             //st.spows[k*4+m]=(float)cdata[2*k];
             st.spows[k*st.numch+m]=(float)cdata[2*k];   // 2018-02-04
#endif
          }
          for(k=0; k<nfft; k++){  // normalize
             //st.spows[k*4+m]/=pmax;
             st.spows[k*st.numch+m]/=pmax;   // 2018-02-04
             // if(k<10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
             // if(k>nfft-10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
          }
             
       }
         
    } else {

        /* convert power to dB  then dBm */       
        cofst=OFFSET1;       //1 bit sampling full power convert to 0 dB
        if( st.adbit==8) cofst=OFFSET8;     // 8bit sampling offset
        if( st.adbit==4) cofst=OFFSET4;     // 4bit sampling offset
        if( st.adbit==2) cofst=OFFSET2;     // 2bit sampling offset
        if( st.adbit==16) cofst=OFFSET16;     // 16bit sampling offset (arbitrary value) added on 2015-10-23
        for(m=0; m<st.numch; m++){
            for(k=0; k<nfft; k++){
                //st.spows[k*4+m]/=scount[m]; // nomalization
                //st.spows[k*4+m]=(float)(10.0*log10(st.spows[k*4+m]) + cofst); 
                st.spows[k*st.numch+m]/=scount[m]; // nomalization                               //2018-02-04
                st.spows[k*st.numch+m]=(float)(10.0*log10(st.spows[k*st.numch+m]) + cofst);      //2018-02-04
                if(k==0){
                    //pmaxdbm=st.spows[k*4+m];
                    //pmindbm=st.spows[k*4+m];
                    pmaxdbm=st.spows[k*st.numch+m];      //2018-02-04
                    pmindbm=st.spows[k*st.numch+m];      //2018-02-04
                    if(m==0){
                        ymaxdb=pmaxdbm;
                        ymindb=pmindbm;
                    }
                } else {
                    //if(st.spows[k*4+m] > pmaxdbm) pmaxdbm=st.spows[k*4+m]; 
                    //if(st.spows[k*4+m] < pmindbm) pmindbm=st.spows[k*4+m]; 
                    if(st.spows[k*st.numch+m] > pmaxdbm) pmaxdbm=st.spows[k*st.numch+m];      //2018-02-04
                    if(st.spows[k*st.numch+m] < pmindbm) pmindbm=st.spows[k*st.numch+m];      //2018-02-04
                }
            }
            pmax=pmaxdbm-cofst;
            pmin=pmindbm-cofst;
            printf("CH# %d: Maximum data (dB, dBm) is %f %f\n", m+1,pmax,pmaxdbm);      
            if(pmaxdbm > ymaxdb) ymaxdb=pmaxdbm;
            if(pmindbm < ymindb) ymindb=pmindbm;
        }
        printf("All CH: Maximum and Minimum data (dBm) are %f %f\n", ymaxdb, ymindb);      

    }
    st.ymaxdb=ymaxdb;  // 2015-11-03
    st.ymindb=ymindb;  // 2015-11-03
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);

    /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/
     plotspe(&st, (float)st.ymaxdb, (float)st.ymindb);

  exitend: ;
      
    free(K5.ubuf);
    free(K5.idata);
    free(cdata);
    free(st.spows);

#ifdef USE_FFTW
    fftw_destroy_plan(pfcc);
    fftw_destroy_plan(pbcc);
    fftw_free(cc);
#endif

 exitend2: ;

    return 0;
}
